#
#     File name: Functions.r
#
#     Functions to compute a block diagonal matrix and
#     to setup B-spline smoothing
#       
#     ================================================
#
#       Function to construct a block diagonal matrix from
#       a list of matrices or vectors.  x is a list of matrices.
#       NULLs and scalars are allowed in the list.
#
dbind <- function(x){
  len.x <- length(x)
  dim.x <- do.call("cbind", lapply(x, function(x)
  if(length(x) == 0) 0 else {
    if(length(x) == 1) c(1, 1) else dim(x)
  }
  ))
  dim.res <- c( sum(dim.x[1,  ]), sum(dim.x[2,  ]) )
  res <- array(0, dim.res)
  res.ends <- cumsum(dim.x[1,  ])
  res.ends2 <- cumsum(dim.x[2,  ])
  res.starts <- c(1, res.ends[ - len.x] + 1)
  res.starts2 <- c(1, res.ends2[ - len.x] + 1)
  for(i in seq(len.x))
  if(dim.x[1, i] > 0) res[res.starts[i]:res.ends[i],
                          res.starts2[i]:res.ends2[i]] <- x[[i]]
  res
}
#
#     ====================================
#
#     Compute B-spline regression matrix.
#
#     Arguments
#
#     X. = x-domain vector
#     XL. = left of x-domain, XR. = right of x-domain
#     DX. = distance between knots
#     BDEG. = degree of B-spline (quadratic = 2, etc)
#
bspline <- function(X., XL., XR., DX., BDEG.){
 knots <- seq(XL. - BDEG.*DX., XR. + BDEG.*DX., by = DX.)
 B <- spline.des(knots, X., BDEG.+1, 0*X.)$design
 return(list(B = B, Knots = knots))
}
#
#     =====================================
#
#     Set up domain for age
#
#     Arguments: AGE.MIN. = minimum age
#                AGE.MAX. = maximum age
#                DX. = interval between age knots
#
Domain.Age <- function(AGE.MIN., AGE.MAX., DX.){
  Domain <- seq(AGE.MIN., AGE.MAX., by = DX.)
  if(max(Domain) < AGE.MAX.) Domain <- c(Domain, max(Domain) + DX.)
  return(list(Domain = Domain))
 }
#
#     =========================================
#
#     Computes B and D'D
#
#     Arguments: as bspline( ).  Also PORD. = order of the penalty
#
setup.P <- function(X., XL., XR., BDEG., PORD., NDX.){
  B <- bspline(X., XL., XR., NDX., BDEG.)
  D <- diag(ncol(B))
  for (k in 1:PORD.) D <- diff(D)
  DtD <- t(D) %*% D
  return(list(B = B, DtD = DtD))
}
